import { createResource, createSignal, Show, For } from "solid-js";
import axios from "axios";
import { fetchUsers, postUser, putUser, deleteUser } from "../stores/userStore";

export default function UserTable() {
  const [users, { refetch }] = createResource(fetchUsers);
  const [showFormModal, setShowFormModal] = createSignal(false);
  const [editMode, setEditMode] = createSignal(false);
  const [currentUser, setCurrentUser] = createSignal(null);

  const [form, setForm] = createSignal({ prename: "", surname: "", age: "", city: "" });
  const [formErrors, setFormErrors] = createSignal({});
  const [showDeleteModal, setShowDeleteModal] = createSignal(false);
  const [userToDelete, setUserToDelete] = createSignal(null);

  const validate = () => {
    const errors = {};
    const { prename, surname, age, city } = form();

    if (!prename || prename.length < 2) errors.prename = "Mindestens 2 Zeichen";
    if (!surname || surname.length < 2) errors.surname = "Mindestens 2 Zeichen";
    if (!city || city.length < 2) errors.city = "Mindestens 2 Zeichen";
    if (!age || isNaN(Number(age))) errors.age = "Alter muss eine Zahl sein";

    setFormErrors(errors);
    return Object.keys(errors).length === 0;
  };

  const openForm = (user = null) => {
    if (user) {
      setForm({ ...user });
      setEditMode(true);
      setCurrentUser(user);
    } else {
      setForm({ prename: "", surname: "", age: "", city: "" });
      setEditMode(false);
      setCurrentUser(null);
    }
    setFormErrors({});
    setShowFormModal(true);
  };

  const submitForm = async () => {
    if (!validate()) return;
    try {
      if (editMode()) {
        await putUser(currentUser().id, form());
      } else {
        await postUser(form());
      }
      setShowFormModal(false);
      refetch();
    } catch (e) {
      alert(e.message);
    }
  };

  const confirmDelete = async () => {
    try {
      await deleteUser(userToDelete().id);
      setShowDeleteModal(false);
      refetch();
    } catch (e) {
      alert(e.message);
    }
  };

  return (
    <div class="overflow-x-auto p-4">
      <div class="flex justify-between items-center mb-4">
        <h2 class="text-2xl font-bold">Benutzerliste</h2>
        <button class="btn btn-primary" onClick={() => openForm()}>
          <span class="text-xl mr-1">+</span> Neuer User
        </button>
      </div>

      <table class="table table-zebra w-full">
        <thead>
          <tr>
            <th>Vorname</th>
            <th>Nachname</th>
            <th>Alter</th>
            <th>Stadt</th>
            <th>Bearbeiten</th>
            <th>Löschen</th>
          </tr>
        </thead>
        <tbody>
          <Show when={users.loading}>
            <tr><td colspan="6" class="text-center">Lädt...</td></tr>
          </Show>
          <Show when={users.error}>
            <tr><td colspan="6" class="text-red-600 font-semibold">{users.error.message}</td></tr>
          </Show>
          <For each={users()}>{(user) => (
            <tr>
              <td>{user.prename}</td>
              <td>{user.surname}</td>
              <td>{user.age}</td>
              <td>{user.city}</td>
              <td>
                <button class="btn btn-sm btn-outline" onClick={() => openForm(user)}>
                  Edit
                </button>
              </td>
              <td>
                <button class="btn btn-sm btn-error" onClick={() => {
                  setUserToDelete(user);
                  setShowDeleteModal(true);
                }}>
                  Delete
                </button>
              </td>
            </tr>
          )}</For>
        </tbody>
      </table>

      {/* Modal: User-Formular */}
      <Show when={showFormModal()}>
        <dialog open class="modal">
          <div class="modal-box space-y-4">
            <h3 class="font-bold text-lg">{editMode() ? "Benutzer bearbeiten" : "Neuer Benutzer"}</h3>

            <label class="input">
              <span class="w-24">Vorname</span>
              <input type="text" class="grow" placeholder ="Vorname" value={form().prename} onInput={(e) => setForm({ ...form(), prename: e.target.value })} />
            </label>
            <Show when={formErrors().prename}><span class="text-error text-sm">{formErrors().prename}</span></Show>

            <label class="input">
              <span class="w-24">Nachname</span>
              <input type="text" class="grow" value={form().surname} onInput={(e) => setForm({ ...form(), surname: e.target.value })} />
            </label>
            <Show when={formErrors().surname}><span class="text-error text-sm">{formErrors().surname}</span></Show>

            <label class="input">
              <span class="w-24">Alter</span>
              <input type="text" class="grow" value={form().age} onInput={(e) => setForm({ ...form(), age: e.target.value })} />
            </label>
            <Show when={formErrors().age}><span class="text-error text-sm">{formErrors().age}</span></Show>

            <label class="input">
              <span class="w-24">Stadt</span>
              <input type="text" class="grow" value={form().city} onInput={(e) => setForm({ ...form(), city: e.target.value })} />
            </label>
            <Show when={formErrors().city}><span class="text-error text-sm">{formErrors().city}</span></Show>

            <div class="modal-action">
              <button type="button" class="btn" onClick={() => setShowFormModal(false)}>Abbrechen</button>
              <button type="button" class="btn btn-primary" onClick={submitForm}>Speichern</button>
            </div>
          </div>
        </dialog>
      </Show>

      {/* Modal: Delete-Bestätigung */}
      <Show when={showDeleteModal()}>
        <dialog open class="modal">
          <div class="modal-box">
            <h3 class="font-bold text-lg mb-4">Benutzer löschen</h3>
            <p>Möchten Sie den Benutzer wirklich löschen?</p>
            <div class="modal-action">
              <button class="btn" onClick={() => setShowDeleteModal(false)}>Abbrechen</button>
              <button class="btn btn-error" onClick={confirmDelete}>Löschen</button>
            </div>
          </div>
        </dialog>
      </Show>
    </div>
  );
}