import { createResource, createSignal, Show, For, Switch, Match } from "solid-js";
import { fetchPersons, postPerson, putPerson, deletePerson } from "../stores/personStore";

export default function UserTable(props) {
  const [persons, { refetch }] = createResource(fetchPersons);
  const [showFormModal, setShowFormModal] = createSignal(false);
  const [isEditMode, setIsEditMode] = createSignal(false);

  const [id, setId] = createSignal(null);
  const [prename, setPrename] = createSignal("");
  const [surname, setSurname] = createSignal("");
  const [age, setAge] = createSignal(null);
  const [city, setCity] = createSignal("");
  const [formErrors, setFormErrors] = createSignal({});
  const [showDeleteModal, setShowDeleteModal] = createSignal(false);
  const [personToDelete, setPersonToDelete] = createSignal(null);

  const validate = () => {
    const errors = {};

    // Überprüfen, ob die Daten valide sind. 
    if (prename().length < 2) errors.prename = "Mindestens 2 Zeichen";
    if (surname().length < 2) errors.surname = "Mindestens 2 Zeichen";
    if (city().length < 2) errors.city = "Mindestens 2 Zeichen";
    if (isNaN(Number(age()))) errors.age = "Alter muss eine Zahl sein";

    setFormErrors(errors);
    // Wenn kein Fehler gefunden wurde dann enthält FormErrors keine Einträge. 
    // Objekt.keys liefert die Keys des Objekts error als Array. 
    return Object.keys(errors).length === 0;
  };

  const openModal = (user = null) => {
    if (user) {
      setId(user.id); 
      setPrename(user.prename);
      setSurname(user.surname);
      setAge(user.age);
      setCity(user.city);
      setIsEditMode(true);
    } else {
      setPrename("");
      setSurname("");
      setAge("");
      setCity("");
      setIsEditMode(false);
    }
    // Fehler zurücksetzen und Modal öffnen. 
    setFormErrors({});
    setShowFormModal(true);
  };

  // Submit Person schickt die Daten zum Server. 
  const submitPerson = async () => {
    // zuerst wird geprüft, ob die Daten valide sind, falls nicht werden die 
    // Fehler angezeigt und es wird nichts verschickt, der Nutzer kann nun 
    // korrigieren. 
    if (!validate()) return;

    // der Try Catch Block hört auf Fehler, da putUser und postUser Fehler werfen können. 
    try {
      // Ein ein neuer Nutzer angelegt werden soll, dann 
      if (isEditMode()) {
        let person = { id: id(), prename: prename(), surname: surname(), age: age(), city: city()}
        await putPerson(person);
      } else {
        let person = { prename: prename(), surname: surname(), age: age(), city: city()}
        await postPerson(person);
      }
      setShowFormModal(false);
      refetch();
    } catch (e) {
      alert(e.message);
    }
  };

  const confirmDelete = async () => {
    try {
      await deletePerson(personToDelete().id);
      setShowDeleteModal(false);
      refetch();
    } catch (e) {
      alert(e.message);
    }
  };

  return (
    <div data-theme={props.theme} class="w-full overflow-x-auto pt-10 p-4">
      <div class="w-full flex justify-between items-center mb-4">
        <h2 class="text-2xl font-bold">Personenliste</h2>
        {/* Wenn openForm keinen Parameter bekommt, wird automatisch "null angegenommen", wegen dem default-Parameter in openForm */}
        <button class="btn btn-primary" onClick={() => openModal()}>
          <span class="text-xl mr-1">+</span> Neuer User
        </button>
      </div>
      <Switch>
            <Match when={persons.loading}>
              <tr><td colspan="6" class="text-center">Lädt...</td></tr>
            </Match>
            <Match when={persons.error}>
              <tr><td colspan="6" class="text-red-600 font-semibold">{persons.error.message}</td></tr>
            </Match>
            <Match when={persons()}>
              <table class="table table-zebra w-full">
                <thead>
                  <tr>
                    <th>Vorname</th>
                    <th>Nachname</th>
                    <th>Alter</th>
                    <th>Stadt</th>
                    <th>Bearbeiten</th>
                    <th>Löschen</th>
                  </tr>
                </thead>
                <tbody>
                      <For each={persons()}>{(person) => (
                        <tr>
                          <td>{person.prename}</td>
                          <td>{person.surname}</td>
                          <td>{person.age}</td>
                          <td>{person.city}</td>
                          <td>
                            <button class="btn btn-sm btn-outline" onClick={() => openModal(person)}>
                              Edit
                            </button>
                          </td>
                          <td>
                            <button class="btn btn-sm btn-error" onClick={() => {
                              setPersonToDelete(person);
                              setShowDeleteModal(true);
                            }}>
                              Delete
                            </button>
                          </td>
                        </tr>
                      )}</For>

                </tbody>
              </table>
         </Match>
      </Switch>
      <Show when={showFormModal()}>
        <dialog open class="modal">
          <div class="modal-box space-y-4">
            {/* Mit isEditMode wurde festgelegt, ob eine Person editiert oder eine neue Person angelegt werden soll. */}
            <h3 class="font-bold text-lg">{isEditMode() ? "Benutzer bearbeiten" : "Neuer Benutzer"}</h3>
            <div class="form-control w-full">
              <label class="label">
                <span class="label-text">Vorname</span>
              </label>
              <input
                type="text"
                placeholder="Vorname"
                class="input input-bordered w-full"
                value={prename()}
                onInput={(e) => setPrename(e.target.value)}
              />
              {/* Nach der Validierung gibt es die Objekte (hier formsError().prename bei Fehlern sonst nicht. */}
              <Show when={formErrors().prename}>
                <span class="text-error text-sm mt-1">{formErrors().prename}</span>
              </Show>
            </div>

            <div class="form-control w-full">
              <label class="label">
                <span class="label-text">Nachname</span>
              </label>
              <input
                type="text"
                placeholder="Nachname"
                class="input input-bordered w-full"
                value={surname()}
                onInput={(e) => setSurname(e.target.value)}
              />
              <Show when={formErrors().surname}>
                <span class="text-error text-sm mt-1">{formErrors().surname}</span>
              </Show>
            </div>

            <div class="form-control w-full">
              <label class="label">
                <span class="label-text">Alter</span>
              </label>
              <input
                type="number"
                placeholder="Alter"
                class="input input-bordered w-full"
                value={age()}
                onInput={(e) => setAge(e.target.value)}
              />
              <Show when={formErrors().age}>
                <span class="text-error text-sm mt-1">{formErrors().age}</span>
              </Show>
            </div>

            <div class="form-control w-full">
              <label class="label">
                <span class="label-text">Stadt</span>
              </label>
              <input
                type="text"
                placeholder="Stadt"
                class="input input-bordered w-full"
                value={city()}
                onInput={(e) => setCity(e.target.value)}
              />
              <Show when={formErrors().city}>
                <span class="text-error text-sm mt-1">{formErrors().city}</span>
              </Show>
            </div>

            {/* Aktionen */}
            <div class="modal-action">
              <button type="button" class="btn" onClick={() => setShowFormModal(false)}>Abbrechen</button>
              <button type="button" class="btn btn-primary" onClick={ submitPerson }>Speichern</button>

            </div>            
          </div>
        </dialog>
      </Show>

      {/* Modal: Delete-Bestätigung */}
      <Show when={showDeleteModal()}>
        <dialog open class="modal">
          <div class="modal-box">
            <h3 class="font-bold text-lg mb-4">Benutzer löschen</h3>
            <p>Möchten Sie den Benutzer wirklich löschen?</p>
            <div class="modal-action">
              <button class="btn" onClick={() => setShowDeleteModal(false)}>Abbrechen</button>
              <button class="btn btn-error" onClick={confirmDelete}>Löschen</button>
            </div>
          </div>
        </dialog>
      </Show>
    </div>
  );
}